@@ -8,23 +8,21 @@
 import sys
 from math import sqrt
 
-error = 'fact.error'            # exception
-
 def fact(n):
-    if n < 1: raise error   # fact() argument should be >= 1
+    if n < 1: raise ValueError # fact() argument should be >= 1
     if n == 1: return []    # special case
     res = []
     # Treat even factors special, so we can use i = i+2 later
     while n%2 == 0:
         res.append(2)
-        n = n/2
+        n = n//2
     # Try odd numbers up to sqrt(n)
     limit = sqrt(float(n+1))
     i = 3
     while i <= limit:
         if n%i == 0:
             res.append(i)
-            n = n/i
+            n = n//i
             limit = sqrt(n+1)
         else:
             i = i+2
@@ -36,12 +34,12 @@ def main():
     if len(sys.argv) > 1:
         for arg in sys.argv[1:]:
             n = eval(arg)
-            print n, fact(n)
+            print(n, fact(n))
     else:
         try:
             while 1:
-                n = input()
-                print n, fact(n)
+                n = eval(input())
+                print(n, fact(n))
         except EOFError:
             pass
 
