@@ -4,10 +4,9 @@ Provides the Distribution class, which represents the module distribution
 being built/installed/distributed.
 """
 
-__revision__ = "$Id: dist.py 65250 2008-07-26 20:09:45Z amaury.forgeotdarc $"
+__revision__ = "$Id: dist.py 73123 2009-06-01 22:36:26Z tarek.ziade $"
 
 import sys, os, re
-from copy import copy
 
 try:
     import warnings
@@ -199,6 +198,7 @@ Common commands: (see '--help-commands' for more)
         self.extra_path = None
         self.scripts = None
         self.data_files = None
+        self.password = ''
 
         # And now initialize bookkeeping stuff that can't be supplied by
         # the caller at all.  'command_obj' maps command names to
@@ -228,7 +228,7 @@ Common commands: (see '--help-commands' for more)
             # command options will override any supplied redundantly
             # through the general options dictionary.
             options = attrs.get('options')
-            if options:
+            if options is not None:
                 del attrs['options']
                 for (command, cmd_options) in options.items():
                     opt_dict = self.get_option_dict(command)
@@ -246,7 +246,7 @@ Common commands: (see '--help-commands' for more)
 
             # Now work on the rest of the attributes.  Any attribute that's
             # not already defined is invalid!
-            for (key,val) in attrs.items():
+            for (key, val) in attrs.items():
                 if hasattr(self.metadata, "set_" + key):
                     getattr(self.metadata, "set_" + key)(val)
                 elif hasattr(self.metadata, key):
@@ -262,49 +262,46 @@ Common commands: (see '--help-commands' for more)
 
         self.finalize_options()
 
-
-    def get_option_dict (self, command):
+    def get_option_dict(self, command):
         """Get the option dictionary for a given command.  If that
         command's option dictionary hasn't been created yet, then create it
         and return the new dictionary; otherwise, return the existing
         option dictionary.
         """
-
         dict = self.command_options.get(command)
         if dict is None:
             dict = self.command_options[command] = {}
         return dict
 
-
-    def dump_option_dicts (self, header=None, commands=None, indent=""):
+    def dump_option_dicts(self, header=None, commands=None, indent=""):
         from pprint import pformat
 
         if commands is None:             # dump all command option dicts
             commands = sorted(self.command_options.keys())
 
         if header is not None:
-            print(indent + header)
+            self.announce(indent + header)
             indent = indent + "  "
 
         if not commands:
-            print(indent + "no commands known yet")
+            self.announce(indent + "no commands known yet")
             return
 
         for cmd_name in commands:
             opt_dict = self.command_options.get(cmd_name)
             if opt_dict is None:
-                print(indent + "no option dict for '%s' command" % cmd_name)
+                self.announce(indent +
+                              "no option dict for '%s' command" % cmd_name)
             else:
-                print(indent + "option dict for '%s' command:" % cmd_name)
+                self.announce(indent +
+                              "option dict for '%s' command:" % cmd_name)
                 out = pformat(opt_dict)
-                for line in out.split("\n"):
-                    print(indent + "  " + line)
-
-
+                for line in out.split('\n'):
+                    self.announce(indent + "  " + line)
 
     # -- Config file finding/parsing methods ---------------------------
 
-    def find_config_files (self):
+    def find_config_files(self):
         """Find as many configuration files as should be processed for this
         platform, and return a list of filenames in the order in which they
         should be parsed.  The filenames returned are guaranteed to exist
@@ -345,19 +342,19 @@ Common commands: (see '--help-commands' for more)
 
         return files
 
-
-    def parse_config_files (self, filenames=None):
-
+    def parse_config_files(self, filenames=None):
         from configparser import ConfigParser
 
         if filenames is None:
             filenames = self.find_config_files()
 
-        if DEBUG: print("Distribution.parse_config_files():")
+        if DEBUG:
+            self.announce("Distribution.parse_config_files():")
 
         parser = ConfigParser()
         for filename in filenames:
-            if DEBUG: print("  reading", filename)
+            if DEBUG:
+                self.announce("  reading", filename)
             parser.read(filename)
             for section in parser.sections():
                 options = parser.options(section)
@@ -389,10 +386,9 @@ Common commands: (see '--help-commands' for more)
                 except ValueError as msg:
                     raise DistutilsOptionError(msg)
 
-
     # -- Command-line parsing methods ----------------------------------
 
-    def parse_command_line (self):
+    def parse_command_line(self):
         """Parse the setup script's command line, taken from the
         'script_args' instance attribute (which defaults to 'sys.argv[1:]'
         -- see 'setup()' in core.py).  This list is first processed for
@@ -416,11 +412,6 @@ Common commands: (see '--help-commands' for more)
         # that allows the user to interactively specify the "command line".
         #
         toplevel_options = self._get_toplevel_options()
-        if sys.platform == 'mac':
-            import EasyDialogs
-            cmdlist = self.get_command_list()
-            self.script_args = EasyDialogs.GetArgv(
-                toplevel_options + self.display_options, cmdlist)
 
         # We have to parse the command line a bit at a time -- global
         # options, then the first command, then its options, and so on --
@@ -440,7 +431,6 @@ Common commands: (see '--help-commands' for more)
         # for display options we return immediately
         if self.handle_display_options(option_order):
             return
-
         while args:
             args = self._parse_command_opts(parser, args)
             if args is None:            # user asked for help (and got it)
@@ -465,7 +455,7 @@ Common commands: (see '--help-commands' for more)
         # All is well: return true
         return True
 
-    def _get_toplevel_options (self):
+    def _get_toplevel_options(self):
         """Return the non-display options recognized at the top level.
 
         This includes options that are recognized *only* at the top
@@ -476,7 +466,7 @@ Common commands: (see '--help-commands' for more)
              "list of packages that provide distutils commands"),
             ]
 
-    def _parse_command_opts (self, parser, args):
+    def _parse_command_opts(self, parser, args):
         """Parse the command-line options for a single command.
         'parser' must be a FancyGetopt instance; 'args' must be the list
         of arguments, starting with the current command (whose options
@@ -520,7 +510,7 @@ Common commands: (see '--help-commands' for more)
         # merge it in with the global negative aliases.
         negative_opt = self.negative_opt
         if hasattr(cmd_class, 'negative_opt'):
-            negative_opt = copy(negative_opt)
+            negative_opt = negative_opt.copy()
             negative_opt.update(cmd_class.negative_opt)
 
         # Check for help_options in command class.  They have a different
@@ -549,9 +539,6 @@ Common commands: (see '--help-commands' for more)
             for (help_option, short, desc, func) in cmd_class.help_options:
                 if hasattr(opts, parser.get_attr_name(help_option)):
                     help_option_found=1
-                    #print "showing help for option %s of command %s" % \
-                    #      (help_option[0],cmd_class)
-
                     if hasattr(func, '__call__'):
                         func()
                     else:
@@ -571,29 +558,21 @@ Common commands: (see '--help-commands' for more)
 
         return args
 
-    def finalize_options (self):
+    def finalize_options(self):
         """Set final values for all the options on the Distribution
         instance, analogous to the .finalize_options() method of Command
         objects.
         """
+        for attr in ('keywords', 'platforms'):
+            value = getattr(self.metadata, attr)
+            if value is None:
+                continue
+            if isinstance(value, str):
+                value = [elm.strip() for elm in value.split(',')]
+                setattr(self.metadata, attr, value)
 
-        keywords = self.metadata.keywords
-        if keywords is not None:
-            if isinstance(keywords, str):
-                keywordlist = keywords.split(',')
-                self.metadata.keywords = [x.strip() for x in keywordlist]
-
-        platforms = self.metadata.platforms
-        if platforms is not None:
-            if isinstance(platforms, str):
-                platformlist = platforms.split(',')
-                self.metadata.platforms = [x.strip() for x in platformlist]
-
-    def _show_help (self,
-                    parser,
-                    global_options=1,
-                    display_options=1,
-                    commands=[]):
+    def _show_help(self, parser, global_options=1, display_options=1,
+                   commands=[]):
         """Show help for the setup script command-line in the form of
         several lists of command-line options.  'parser' should be a
         FancyGetopt instance; do not expect it to be returned in the
@@ -617,14 +596,14 @@ Common commands: (see '--help-commands' for more)
                 options = self.global_options
             parser.set_option_table(options)
             parser.print_help(self.common_usage + "\nGlobal options:")
-            print()
+            self.announce('')
 
         if display_options:
             parser.set_option_table(self.display_options)
             parser.print_help(
                 "Information display options (just display " +
                 "information, ignore any commands)")
-            print()
+            self.announce('')
 
         for command in self.commands:
             if isinstance(command, type) and issubclass(command, Command):
@@ -638,13 +617,11 @@ Common commands: (see '--help-commands' for more)
             else:
                 parser.set_option_table(klass.user_options)
             parser.print_help("Options for '%s' command:" % klass.__name__)
-            print()
+            self.announce('')
 
-        print(gen_usage(self.script_name))
-        return
+        self.announce(gen_usage(self.script_name))
 
-
-    def handle_display_options (self, option_order):
+    def handle_display_options(self, option_order):
         """If there were any non-global "display-only" options
         (--help-commands or the metadata display options) on the command
         line, display the requested info and return true; else return
@@ -657,8 +634,8 @@ Common commands: (see '--help-commands' for more)
         # we ignore "foo bar").
         if self.help_commands:
             self.print_commands()
-            print()
-            print(gen_usage(self.script_name))
+            self.announce('')
+            self.announce(gen_usage(self.script_name))
             return 1
 
         # If user supplied any of the "display metadata" options, then
@@ -674,21 +651,21 @@ Common commands: (see '--help-commands' for more)
                 opt = translate_longopt(opt)
                 value = getattr(self.metadata, "get_"+opt)()
                 if opt in ['keywords', 'platforms']:
-                    print(','.join(value))
+                    self.announce(','.join(value))
                 elif opt in ('classifiers', 'provides', 'requires',
                              'obsoletes'):
-                    print('\n'.join(value))
+                    self.announce('\n'.join(value))
                 else:
-                    print(value)
+                    self.announce(value)
                 any_display_options = 1
 
         return any_display_options
 
-    def print_command_list (self, commands, header, max_length):
+    def print_command_list(self, commands, header, max_length):
         """Print a subset of the list of all commands -- used by
         'print_commands()'.
         """
-        print(header + ":")
+        self.announce(header + ":")
 
         for cmd in commands:
             klass = self.cmdclass.get(cmd)
@@ -699,10 +676,9 @@ Common commands: (see '--help-commands' for more)
             except AttributeError:
                 description = "(no description available)"
 
-            print("  %-*s  %s" % (max_length, cmd, description))
+            self.announce("  %-*s  %s" % (max_length, cmd, description))
 
-
-    def print_commands (self):
+    def print_commands(self):
         """Print out a help message listing all available commands with a
         description of each.  The list is divided into "standard commands"
         (listed in distutils.command.__all__) and "extra commands"
@@ -735,7 +711,7 @@ Common commands: (see '--help-commands' for more)
                                     "Extra commands",
                                     max_length)
 
-    def get_command_list (self):
+    def get_command_list(self):
         """Get a list of (command, description) tuples.
         The list is divided into "standard commands" (listed in
         distutils.command.__all__) and "extra commands" (mentioned in
@@ -769,20 +745,19 @@ Common commands: (see '--help-commands' for more)
 
     # -- Command class/object methods ----------------------------------
 
-    def get_command_packages (self):
+    def get_command_packages(self):
         """Return a list of packages from which commands are loaded."""
         pkgs = self.command_packages
-        if not isinstance(pkgs, type([])):
-            pkgs = (pkgs or "").split(",")
-            for i in range(len(pkgs)):
-                pkgs[i] = pkgs[i].strip()
-            pkgs = [p for p in pkgs if p]
+        if not isinstance(pkgs, list):
+            if pkgs is None:
+                pkgs = ''
+            pkgs = [pkg.strip() for pkg in pkgs.split(',') if pkg != '']
             if "distutils.command" not in pkgs:
                 pkgs.insert(0, "distutils.command")
             self.command_packages = pkgs
         return pkgs
 
-    def get_command_class (self, command):
+    def get_command_class(self, command):
         """Return the class that implements the Distutils command named by
         'command'.  First we check the 'cmdclass' dictionary; if the
         command is mentioned there, we fetch the class object from the
@@ -820,7 +795,7 @@ Common commands: (see '--help-commands' for more)
 
         raise DistutilsModuleError("invalid command '%s'" % command)
 
-    def get_command_obj (self, command, create=1):
+    def get_command_obj(self, command, create=1):
         """Return the command object for 'command'.  Normally this object
         is cached on a previous call to 'get_command_obj()'; if no command
         object for 'command' is in the cache, then we either create and
@@ -829,8 +804,8 @@ Common commands: (see '--help-commands' for more)
         cmd_obj = self.command_obj.get(command)
         if not cmd_obj and create:
             if DEBUG:
-                print("Distribution.get_command_obj(): " \
-                      "creating '%s' command object" % command)
+                self.announce("Distribution.get_command_obj(): " \
+                              "creating '%s' command object" % command)
 
             klass = self.get_command_class(command)
             cmd_obj = self.command_obj[command] = klass(self)
@@ -847,7 +822,7 @@ Common commands: (see '--help-commands' for more)
 
         return cmd_obj
 
-    def _set_command_options (self, command_obj, option_dict=None):
+    def _set_command_options(self, command_obj, option_dict=None):
         """Set the options for 'command_obj' from 'option_dict'.  Basically
         this means copying elements of a dictionary ('option_dict') to
         attributes of an instance ('command').
@@ -860,9 +835,12 @@ Common commands: (see '--help-commands' for more)
         if option_dict is None:
             option_dict = self.get_option_dict(command_name)
 
-        if DEBUG: print("  setting options for '%s' command:" % command_name)
+        if DEBUG:
+            self.announce("  setting options for '%s' command:" % command_name)
         for (option, (source, value)) in option_dict.items():
-            if DEBUG: print("    %s = %s (from %s)" % (option, value, source))
+            if DEBUG:
+                self.announce("    %s = %s (from %s)" % (option, value,
+                                                         source))
             try:
                 bool_opts = [translate_longopt(o)
                              for o in command_obj.boolean_options]
@@ -888,7 +866,7 @@ Common commands: (see '--help-commands' for more)
             except ValueError as msg:
                 raise DistutilsOptionError(msg)
 
-    def reinitialize_command (self, command, reinit_subcommands=0):
+    def reinitialize_command(self, command, reinit_subcommands=0):
         """Reinitializes a command to the state it was in when first
         returned by 'get_command_obj()': ie., initialized but not yet
         finalized.  This provides the opportunity to sneak option
@@ -927,13 +905,12 @@ Common commands: (see '--help-commands' for more)
 
         return command
 
-
     # -- Methods that operate on the Distribution ----------------------
 
-    def announce (self, msg, level=1):
+    def announce(self, msg, level=1):
         log.debug(msg)
 
-    def run_commands (self):
+    def run_commands(self):
         """Run each command that was seen on the setup script command line.
         Uses the list of commands found and cache of command objects
         created by 'get_command_obj()'.
@@ -941,10 +918,9 @@ Common commands: (see '--help-commands' for more)
         for cmd in self.commands:
             self.run_command(cmd)
 
-
     # -- Methods that operate on its Commands --------------------------
 
-    def run_command (self, command):
+    def run_command(self, command):
         """Do whatever it takes to run a command (including nothing at all,
         if the command has already been run).  Specifically: if we have
         already created and run the command named by 'command', return
@@ -965,28 +941,28 @@ Common commands: (see '--help-commands' for more)
 
     # -- Distribution query methods ------------------------------------
 
-    def has_pure_modules (self):
+    def has_pure_modules(self):
         return len(self.packages or self.py_modules or []) > 0
 
-    def has_ext_modules (self):
+    def has_ext_modules(self):
         return self.ext_modules and len(self.ext_modules) > 0
 
-    def has_c_libraries (self):
+    def has_c_libraries(self):
         return self.libraries and len(self.libraries) > 0
 
-    def has_modules (self):
+    def has_modules(self):
         return self.has_pure_modules() or self.has_ext_modules()
 
-    def has_headers (self):
+    def has_headers(self):
         return self.headers and len(self.headers) > 0
 
-    def has_scripts (self):
+    def has_scripts(self):
         return self.scripts and len(self.scripts) > 0
 
-    def has_data_files (self):
+    def has_data_files(self):
         return self.data_files and len(self.data_files) > 0
 
-    def is_pure (self):
+    def is_pure(self):
         return (self.has_pure_modules() and
                 not self.has_ext_modules() and
                 not self.has_c_libraries())
@@ -998,9 +974,6 @@ Common commands: (see '--help-commands' for more)
     # to self.metadata.get_XXX.  The actual code is in the
     # DistributionMetadata class, below.
 
-# class Distribution
-
-
 class DistributionMetadata:
     """Dummy class to hold the distribution meta-data: name, version,
     author, and so forth.
@@ -1036,16 +1009,14 @@ class DistributionMetadata:
         self.requires = None
         self.obsoletes = None
 
-    def write_pkg_info (self, base_dir):
+    def write_pkg_info(self, base_dir):
         """Write the PKG-INFO file into the release tree.
         """
         pkg_info = open( os.path.join(base_dir, 'PKG-INFO'), 'w')
-
         self.write_pkg_file(pkg_info)
-
         pkg_info.close()
 
-    def write_pkg_file (self, file):
+    def write_pkg_file(self, file):
         """Write the PKG-INFO format data to a file object.
         """
         version = '1.0'
@@ -1063,7 +1034,7 @@ class DistributionMetadata:
         if self.download_url:
             file.write('Download-URL: %s\n' % self.download_url)
 
-        long_desc = rfc822_escape( self.get_long_description() )
+        long_desc = rfc822_escape(self.get_long_description())
         file.write('Description: %s\n' % long_desc)
 
         keywords = ','.join(self.get_keywords())
@@ -1078,19 +1049,19 @@ class DistributionMetadata:
         self._write_list(file, 'Provides', self.get_provides())
         self._write_list(file, 'Obsoletes', self.get_obsoletes())
 
-    def _write_list (self, file, name, values):
+    def _write_list(self, file, name, values):
         for value in values:
             file.write('%s: %s\n' % (name, value))
 
     # -- Metadata query methods ----------------------------------------
 
-    def get_name (self):
+    def get_name(self):
         return self.name or "UNKNOWN"
 
     def get_version(self):
         return self.version or "0.0.0"
 
-    def get_fullname (self):
+    def get_fullname(self):
         return "%s-%s" % (self.get_name(), self.get_version())
 
     def get_author(self):
@@ -1106,14 +1077,10 @@ class DistributionMetadata:
         return self.maintainer_email or "UNKNOWN"
 
     def get_contact(self):
-        return (self.maintainer or
-                self.author or
-                "UNKNOWN")
+        return self.maintainer or self.author or "UNKNOWN"
 
     def get_contact_email(self):
-        return (self.maintainer_email or
-                self.author_email or
-                "UNKNOWN")
+        return self.maintainer_email or self.author_email or "UNKNOWN"
 
     def get_url(self):
         return self.url or "UNKNOWN"
@@ -1141,7 +1108,6 @@ class DistributionMetadata:
         return self.download_url or "UNKNOWN"
 
     # PEP 314
-
     def get_requires(self):
         return self.requires or []
 
@@ -1170,8 +1136,7 @@ class DistributionMetadata:
             distutils.versionpredicate.VersionPredicate(v)
         self.obsoletes = value
 
-
-def fix_help_options (options):
+def fix_help_options(options):
     """Convert a 4-tuple 'help_options' list as found in various command
     classes to the 3-tuple form required by FancyGetopt.
     """
@@ -1179,8 +1144,3 @@ def fix_help_options (options):
     for help_tuple in options:
         new_options.append(help_tuple[0:3])
     return new_options
-
-
-if __name__ == "__main__":
-    dist = Distribution()
-    print("ok")
