@@ -29,12 +29,12 @@
 """Read from and write to tar format archives.
 """
 
-__version__ = "$Revision: 65860 $"
+__version__ = "$Revision: 70525 $"
 
 version     = "0.9.0"
 __author__  = "Lars Gust\u00e4bel (lars@gustaebel.de)"
-__date__    = "$Date: 2008-08-19 13:56:33 -0400 (Tue, 19 Aug 2008) $"
-__cvsid__   = "$Id: tarfile.py 65860 2008-08-19 17:56:33Z antoine.pitrou $"
+__date__    = "$Date: 2009-03-22 15:34:29 -0500 (Sun, 22 Mar 2009) $"
+__cvsid__   = "$Id: tarfile.py 70525 2009-03-22 20:34:29Z lars.gustaebel $"
 __credits__ = "Gustavo Niemeyer, Niels Gust\u00e4bel, Richard Townsend."
 
 #---------
@@ -639,12 +639,11 @@ class _BZ2Proxy(object):
     def read(self, size):
         x = len(self.buf)
         while x < size:
-            try:
-                raw = self.fileobj.read(self.blocksize)
-                data = self.bz2obj.decompress(raw)
-                self.buf += data
-            except EOFError:
+            raw = self.fileobj.read(self.blocksize)
+            if not raw:
                 break
+            data = self.bz2obj.decompress(raw)
+            self.buf += data
             x += len(data)
 
         buf = self.buf[:size]
@@ -1753,7 +1752,7 @@ class TarFile(object):
     def getmember(self, name):
         """Return a TarInfo object for member `name'. If `name' can not be
            found in the archive, KeyError is raised. If a member occurs more
-           than once in the archive, its last occurence is assumed to be the
+           than once in the archive, its last occurrence is assumed to be the
            most up-to-date version.
         """
         tarinfo = self._getmember(name)
@@ -2265,10 +2264,6 @@ class TarFile(object):
         """
         if not hasattr(os, 'utime'):
             return
-        if sys.platform == "win32" and tarinfo.isdir():
-            # According to msdn.microsoft.com, it is an error (EACCES)
-            # to use utime() on directories.
-            return
         try:
             os.utime(targetpath, (tarinfo.mtime, tarinfo.mtime))
         except EnvironmentError as e:
