@@ -1,74 +1,86 @@
 import unittest
-from test import test_support
-import zlib
+from test import support
+import binascii
 import random
 
-# print test_support.TESTFN
-
-def getbuf():
-    # This was in the original.  Avoid non-repeatable sources.
-    # Left here (unused) in case something wants to be done with it.
-    import imp
-    try:
-        t = imp.find_module('test_zlib')
-        file = t[0]
-    except ImportError:
-        file = open(__file__)
-    buf = file.read() * 8
-    file.close()
-    return buf
-
+zlib = support.import_module('zlib')
 
 
 class ChecksumTestCase(unittest.TestCase):
     # checksum test cases
     def test_crc32start(self):
-        self.assertEqual(zlib.crc32(""), zlib.crc32("", 0))
+        self.assertEqual(zlib.crc32(b""), zlib.crc32(b"", 0))
+        self.assert_(zlib.crc32(b"abc", 0xffffffff))
 
     def test_crc32empty(self):
-        self.assertEqual(zlib.crc32("", 0), 0)
-        self.assertEqual(zlib.crc32("", 1), 1)
-        self.assertEqual(zlib.crc32("", 432), 432)
+        self.assertEqual(zlib.crc32(b"", 0), 0)
+        self.assertEqual(zlib.crc32(b"", 1), 1)
+        self.assertEqual(zlib.crc32(b"", 432), 432)
 
     def test_adler32start(self):
-        self.assertEqual(zlib.adler32(""), zlib.adler32("", 1))
+        self.assertEqual(zlib.adler32(b""), zlib.adler32(b"", 1))
+        self.assert_(zlib.adler32(b"abc", 0xffffffff))
 
     def test_adler32empty(self):
-        self.assertEqual(zlib.adler32("", 0), 0)
-        self.assertEqual(zlib.adler32("", 1), 1)
-        self.assertEqual(zlib.adler32("", 432), 432)
+        self.assertEqual(zlib.adler32(b"", 0), 0)
+        self.assertEqual(zlib.adler32(b"", 1), 1)
+        self.assertEqual(zlib.adler32(b"", 432), 432)
 
     def assertEqual32(self, seen, expected):
         # 32-bit values masked -- checksums on 32- vs 64- bit machines
         # This is important if bit 31 (0x08000000L) is set.
-        self.assertEqual(seen & 0x0FFFFFFFFL, expected & 0x0FFFFFFFFL)
+        self.assertEqual(seen & 0x0FFFFFFFF, expected & 0x0FFFFFFFF)
 
     def test_penguins(self):
-        self.assertEqual32(zlib.crc32("penguin", 0), 0x0e5c1a120L)
-        self.assertEqual32(zlib.crc32("penguin", 1), 0x43b6aa94)
-        self.assertEqual32(zlib.adler32("penguin", 0), 0x0bcf02f6)
-        self.assertEqual32(zlib.adler32("penguin", 1), 0x0bd602f7)
+        self.assertEqual32(zlib.crc32(b"penguin", 0), 0x0e5c1a120)
+        self.assertEqual32(zlib.crc32(b"penguin", 1), 0x43b6aa94)
+        self.assertEqual32(zlib.adler32(b"penguin", 0), 0x0bcf02f6)
+        self.assertEqual32(zlib.adler32(b"penguin", 1), 0x0bd602f7)
 
-        self.assertEqual(zlib.crc32("penguin"), zlib.crc32("penguin", 0))
-        self.assertEqual(zlib.adler32("penguin"),zlib.adler32("penguin",1))
+        self.assertEqual(zlib.crc32(b"penguin"), zlib.crc32(b"penguin", 0))
+        self.assertEqual(zlib.adler32(b"penguin"),zlib.adler32(b"penguin",1))
+
+    def test_crc32_adler32_unsigned(self):
+        foo = 'abcdefghijklmnop'
+        # explicitly test signed behavior
+        self.assertEqual(zlib.crc32(foo), 2486878355)
+        self.assertEqual(zlib.crc32('spam'), 1138425661)
+        self.assertEqual(zlib.adler32(foo+foo), 3573550353)
+        self.assertEqual(zlib.adler32('spam'), 72286642)
+
+    def test_same_as_binascii_crc32(self):
+        foo = b'abcdefghijklmnop'
+        crc = 2486878355
+        self.assertEqual(binascii.crc32(foo), crc)
+        self.assertEqual(zlib.crc32(foo), crc)
+        self.assertEqual(binascii.crc32(b'spam'), zlib.crc32(b'spam'))
 
 
 
 class ExceptionTestCase(unittest.TestCase):
     # make sure we generate some expected errors
-    def test_bigbits(self):
-        # specifying total bits too large causes an error
-        self.assertRaises(zlib.error,
-                zlib.compress, 'ERROR', zlib.MAX_WBITS + 1)
+    def test_badlevel(self):
+        # specifying compression level out of range causes an error
+        # (but -1 is Z_DEFAULT_COMPRESSION and apparently the zlib
+        # accepts 0 too)
+        self.assertRaises(zlib.error, zlib.compress, 'ERROR', 10)
 
     def test_badcompressobj(self):
         # verify failure on building compress object with bad params
         self.assertRaises(ValueError, zlib.compressobj, 1, zlib.DEFLATED, 0)
+        # specifying total bits too large causes an error
+        self.assertRaises(ValueError,
+                zlib.compressobj, 1, zlib.DEFLATED, zlib.MAX_WBITS + 1)
 
     def test_baddecompressobj(self):
         # verify failure on building decompress object with bad params
         self.assertRaises(ValueError, zlib.decompressobj, 0)
 
+    def test_decompressobj_badflush(self):
+        # verify failure on calling decompressobj.flush with bad params
+        self.assertRaises(ValueError, zlib.decompressobj().flush, 0)
+        self.assertRaises(ValueError, zlib.decompressobj().flush, -1)
+
 
 
 class CompressTestCase(unittest.TestCase):
@@ -99,6 +111,8 @@ class CompressObjectTestCase(unittest.TestCase):
         y1 = dco.decompress(x1 + x2)
         y2 = dco.flush()
         self.assertEqual(data, y1 + y2)
+        self.assert_(isinstance(dco.unconsumed_tail, bytes))
+        self.assert_(isinstance(dco.unused_data, bytes))
 
     def test_compressoptions(self):
         # specify lots of options to compressobj()
@@ -123,10 +137,10 @@ class CompressObjectTestCase(unittest.TestCase):
         for i in range(0, len(data), 256):
             bufs.append(co.compress(data[i:i+256]))
         bufs.append(co.flush())
-        combuf = ''.join(bufs)
+        combuf = b''.join(bufs)
 
         dco = zlib.decompressobj()
-        y1 = dco.decompress(''.join(bufs))
+        y1 = dco.decompress(b''.join(bufs))
         y2 = dco.flush()
         self.assertEqual(data, y1 + y2)
 
@@ -139,17 +153,22 @@ class CompressObjectTestCase(unittest.TestCase):
         for i in range(0, len(data), cx):
             bufs.append(co.compress(data[i:i+cx]))
         bufs.append(co.flush())
-        combuf = ''.join(bufs)
+        combuf = b''.join(bufs)
 
-        self.assertEqual(data, zlib.decompress(combuf))
+        decombuf = zlib.decompress(combuf)
+        # Test type of return value
+        self.assert_(isinstance(decombuf, bytes))
+
+        self.assertEqual(data, decombuf)
 
         dco = zlib.decompressobj()
         bufs = []
         for i in range(0, len(combuf), dcx):
             bufs.append(dco.decompress(combuf[i:i+dcx]))
-            self.assertEqual('', dco.unconsumed_tail, ########
-                             "(A) uct should be '': not %d long" %
+            self.assertEqual(b'', dco.unconsumed_tail, ########
+                             "(A) uct should be b'': not %d long" %
                                        len(dco.unconsumed_tail))
+            self.assertEqual(b'', dco.unused_data)
         if flush:
             bufs.append(dco.flush())
         else:
@@ -159,10 +178,11 @@ class CompressObjectTestCase(unittest.TestCase):
                     bufs.append(chunk)
                 else:
                     break
-        self.assertEqual('', dco.unconsumed_tail, ########
-                         "(B) uct should be '': not %d long" %
+        self.assertEqual(b'', dco.unconsumed_tail, ########
+                         "(B) uct should be b'': not %d long" %
                                        len(dco.unconsumed_tail))
-        self.assertEqual(data, ''.join(bufs))
+        self.assertEqual(b'', dco.unused_data)
+        self.assertEqual(data, b''.join(bufs))
         # Failure means: "decompressobj with init options failed"
 
     def test_decompincflush(self):
@@ -178,7 +198,7 @@ class CompressObjectTestCase(unittest.TestCase):
         for i in range(0, len(data), cx):
             bufs.append(co.compress(data[i:i+cx]))
         bufs.append(co.flush())
-        combuf = ''.join(bufs)
+        combuf = b''.join(bufs)
         self.assertEqual(data, zlib.decompress(combuf),
                          'compressed data failure')
 
@@ -193,7 +213,7 @@ class CompressObjectTestCase(unittest.TestCase):
             bufs.append(chunk)
             cb = dco.unconsumed_tail
         bufs.append(dco.flush())
-        self.assertEqual(data, ''.join(bufs), 'Wrong data retrieved')
+        self.assertEqual(data, b''.join(bufs), 'Wrong data retrieved')
 
     def test_decompressmaxlen(self, flush=False):
         # Check a decompression object with max_length specified
@@ -203,7 +223,7 @@ class CompressObjectTestCase(unittest.TestCase):
         for i in range(0, len(data), 256):
             bufs.append(co.compress(data[i:i+256]))
         bufs.append(co.flush())
-        combuf = ''.join(bufs)
+        combuf = b''.join(bufs)
         self.assertEqual(data, zlib.decompress(combuf),
                          'compressed data failure')
 
@@ -225,7 +245,7 @@ class CompressObjectTestCase(unittest.TestCase):
                 self.failIf(len(chunk) > max_length,
                             'chunk too big (%d>%d)' % (len(chunk),max_length))
                 bufs.append(chunk)
-        self.assertEqual(data, ''.join(bufs), 'Wrong data retrieved')
+        self.assertEqual(data, b''.join(bufs), 'Wrong data retrieved')
 
     def test_decompressmaxlenflush(self):
         self.test_decompressmaxlen(flush=True)
@@ -234,7 +254,7 @@ class CompressObjectTestCase(unittest.TestCase):
         # Misc tests of max_length
         dco = zlib.decompressobj()
         self.assertRaises(ValueError, dco.decompress, "", -1)
-        self.assertEqual('', dco.unconsumed_tail)
+        self.assertEqual(b'', dco.unconsumed_tail)
 
     def test_flushes(self):
         # Test flush() with the various options, using all the
@@ -251,7 +271,7 @@ class CompressObjectTestCase(unittest.TestCase):
                 b = obj.flush( sync )
                 c = obj.compress( data[3000:] )
                 d = obj.flush()
-                self.assertEqual(zlib.decompress(''.join([a,b,c,d])),
+                self.assertEqual(zlib.decompress(b''.join([a,b,c,d])),
                                  data, ("Decompress failed: flush "
                                         "mode=%i, level=%i") % (sync, level))
                 del obj
@@ -290,6 +310,77 @@ class CompressObjectTestCase(unittest.TestCase):
             # if decompressed data is different from the input data, choke.
             self.assertEqual(expanded, data, "17K random source doesn't match")
 
+    def test_empty_flush(self):
+        # Test that calling .flush() on unused objects works.
+        # (Bug #1083110 -- calling .flush() on decompress objects
+        # caused a core dump.)
+
+        co = zlib.compressobj(zlib.Z_BEST_COMPRESSION)
+        self.failUnless(co.flush())  # Returns a zlib header
+        dco = zlib.decompressobj()
+        self.assertEqual(dco.flush(), b"") # Returns nothing
+
+    if hasattr(zlib.compressobj(), "copy"):
+        def test_compresscopy(self):
+            # Test copying a compression object
+            data0 = HAMLET_SCENE
+            data1 = bytes(str(HAMLET_SCENE, "ascii").swapcase(), "ascii")
+            c0 = zlib.compressobj(zlib.Z_BEST_COMPRESSION)
+            bufs0 = []
+            bufs0.append(c0.compress(data0))
+
+            c1 = c0.copy()
+            bufs1 = bufs0[:]
+
+            bufs0.append(c0.compress(data0))
+            bufs0.append(c0.flush())
+            s0 = b''.join(bufs0)
+
+            bufs1.append(c1.compress(data1))
+            bufs1.append(c1.flush())
+            s1 = b''.join(bufs1)
+
+            self.assertEqual(zlib.decompress(s0),data0+data0)
+            self.assertEqual(zlib.decompress(s1),data0+data1)
+
+        def test_badcompresscopy(self):
+            # Test copying a compression object in an inconsistent state
+            c = zlib.compressobj()
+            c.compress(HAMLET_SCENE)
+            c.flush()
+            self.assertRaises(ValueError, c.copy)
+
+    if hasattr(zlib.decompressobj(), "copy"):
+        def test_decompresscopy(self):
+            # Test copying a decompression object
+            data = HAMLET_SCENE
+            comp = zlib.compress(data)
+            # Test type of return value
+            self.assert_(isinstance(comp, bytes))
+
+            d0 = zlib.decompressobj()
+            bufs0 = []
+            bufs0.append(d0.decompress(comp[:32]))
+
+            d1 = d0.copy()
+            bufs1 = bufs0[:]
+
+            bufs0.append(d0.decompress(comp[32:]))
+            s0 = b''.join(bufs0)
+
+            bufs1.append(d1.decompress(comp[32:]))
+            s1 = b''.join(bufs1)
+
+            self.assertEqual(s0,s1)
+            self.assertEqual(s0,data)
+
+        def test_baddecompresscopy(self):
+            # Test copying a compression object in an inconsistent state
+            data = zlib.compress(HAMLET_SCENE)
+            d = zlib.decompressobj()
+            d.decompress(data)
+            d.flush()
+            self.assertRaises(ValueError, d.copy)
 
 def genblock(seed, length, step=1024, generator=random):
     """length-byte stream of random data from a seed (in step-byte blocks)."""
@@ -298,11 +389,10 @@ def genblock(seed, length, step=1024, generator=random):
     randint = generator.randint
     if length < step or step < 2:
         step = length
-    blocks = []
+    blocks = bytes()
     for i in range(0, length, step):
-        blocks.append(''.join([chr(randint(0,255))
-                               for x in range(step)]))
-    return ''.join(blocks)[:length]
+        blocks += bytes(randint(0, 255) for x in range(step))
+    return blocks
 
 
 
@@ -315,7 +405,7 @@ def choose_lines(source, number, seed=None, generator=random):
 
 
 
-HAMLET_SCENE = """
+HAMLET_SCENE = b"""
 LAERTES
 
        O, fear me not.
@@ -381,7 +471,7 @@ LAERTES
 
 
 def test_main():
-    test_support.run_unittest(
+    support.run_unittest(
         ChecksumTestCase,
         ExceptionTestCase,
         CompressTestCase,
@@ -389,22 +479,5 @@ def test_main():
     )
 
 if __name__ == "__main__":
-    test_main()
-
-def test(tests=''):
-    if not tests: tests = 'o'
-    testcases = []
-    if 'k' in tests: testcases.append(ChecksumTestCase)
-    if 'x' in tests: testcases.append(ExceptionTestCase)
-    if 'c' in tests: testcases.append(CompressTestCase)
-    if 'o' in tests: testcases.append(CompressObjectTestCase)
-    test_support.run_unittest(*testcases)
-
-if False:
-    import sys
-    sys.path.insert(1, '/Py23Src/python/dist/src/Lib/test')
-    import test_zlib as tz
-    ts, ut = tz.test_support, tz.unittest
-    su = ut.TestSuite()
-    su.addTest(ut.makeSuite(tz.CompressTestCase))
-    ts.run_suite(su)
+    unittest.main() # XXX
+    ###test_main()
